From 5fe0109b3dd474cca46b11bc9c4730bdd9ac327d Mon Sep 17 00:00:00 2001
From: Sebastian Poeplau <sebastian.poeplau@eurecom.fr>
Date: Tue, 4 Jun 2019 17:03:45 +0200
Subject: [PATCH] Add a plugin to terminate execution states just after forking

The Lua API only lets us inhibit forking; we would like to see the queries
associated with the fork but terminate the new execution state immediately
afterwards.
---
 src/CMakeLists.txt                        |  3 +++
 src/s2e/Plugins/TerminateForkedStates.cpp | 31 +++++++++++++++++++++++
 src/s2e/Plugins/TerminateForkedStates.h   | 26 +++++++++++++++++++
 3 files changed, 60 insertions(+)
 create mode 100644 src/s2e/Plugins/TerminateForkedStates.cpp
 create mode 100644 src/s2e/Plugins/TerminateForkedStates.h

diff --git a/src/CMakeLists.txt b/src/CMakeLists.txt
index 6d4d1ef..07b0432 100644
--- a/src/CMakeLists.txt
+++ b/src/CMakeLists.txt
@@ -119,6 +119,9 @@ add_library(
     s2e/Plugins/Lua/LuaFunctionAnnotationState.cpp
     s2e/Plugins/Lua/LuaExpression.cpp
     s2e/Plugins/Lua/LuaCoreEvents.cpp
+
+    # SP: Custom plugins
+    s2e/Plugins/TerminateForkedStates.cpp
 )
 
 
diff --git a/src/s2e/Plugins/TerminateForkedStates.cpp b/src/s2e/Plugins/TerminateForkedStates.cpp
new file mode 100644
index 0000000..929fd1d
--- /dev/null
+++ b/src/s2e/Plugins/TerminateForkedStates.cpp
@@ -0,0 +1,31 @@
+#include "TerminateForkedStates.h"
+
+#include <s2e/S2E.h>
+
+namespace s2e {
+namespace plugins {
+
+S2E_DEFINE_PLUGIN(TerminateForkedStates,
+                  "Terminate forked states immediately",
+                  "TerminateForkedStates");
+
+void TerminateForkedStates::initialize() {
+    s2e()->getCorePlugin()->onStateFork.connect(
+        sigc::mem_fun(*this, &TerminateForkedStates::handleStateFork));
+}
+
+void TerminateForkedStates::handleStateFork(S2EExecutionState* original,
+                                            const std::vector<S2EExecutionState*>& newStates,
+                                            const std::vector<klee::ref<klee::Expr>>& newConditions)
+{
+    for (auto state : newStates) {
+        if (state == original)
+            continue;
+
+        s2e()->getExecutor()->terminateStateEarly(
+            *state, "Concolic execution - only following the original path");
+    }
+}
+
+} // namespace plugins
+} // namespace s2e
diff --git a/src/s2e/Plugins/TerminateForkedStates.h b/src/s2e/Plugins/TerminateForkedStates.h
new file mode 100644
index 0000000..1cda635
--- /dev/null
+++ b/src/s2e/Plugins/TerminateForkedStates.h
@@ -0,0 +1,26 @@
+#ifndef TERMINATEFORKEDSTATES_H
+#define TERMINATEFORKEDSTATES_H
+
+#include <s2e/Plugin.h>
+#include <s2e/CorePlugin.h>
+#include <s2e/S2EExecutionState.h>
+
+namespace s2e {
+namespace plugins {
+
+class TerminateForkedStates : public Plugin {
+    S2E_PLUGIN
+
+public:
+    TerminateForkedStates(S2E *s2e) : Plugin(s2e) { }
+
+    void initialize();
+    void handleStateFork(S2EExecutionState* /* original state */,
+                         const std::vector<S2EExecutionState*>& /* new states */,
+                         const std::vector<klee::ref<klee::Expr>>& /* new conditions */);
+};
+
+} // namespace plugins
+} // namespace s2e
+
+#endif
-- 
2.23.0

